import { resolveHTTPResponse } from '@trpc/server/http';
/**
 * Create a SvelteKit handle function for tRPC requests.
 *
 * If you want to use it in conjunction with other SvelteKit handles,
 * consider [the sequence helper function](https://kit.svelte.dev/docs/modules#sveltejs-kit-hooks).
 * @see https://kit.svelte.dev/docs/hooks
 */
export function createTRPCHandle({ router, url = '/trpc', createContext, responseMeta, onError }) {
    return async ({ event, resolve }) => {
        if (event.url.pathname.startsWith(url + '/')) {
            const request = event.request;
            const req = {
                method: request.method,
                headers: request.headers,
                query: event.url.searchParams,
                body: await request.text()
            };
            // Using the default `event.setHeaders` and `event.cookies` will not work
            // as the event in not resolved by SvelteKit. Instead, we "proxy" the access
            // to the headers.
            const originalSetHeaders = event.setHeaders;
            const headersProxy = {};
            event.setHeaders = (headers) => {
                for (const [key, value] of Object.entries(headers)) {
                    headersProxy[key] = value;
                }
                // Still call the original `event.setHeaders` function, as it may be used in SvelteKit internals.
                originalSetHeaders(headers);
            };
            const httpResponse = await resolveHTTPResponse({
                router,
                req,
                path: event.url.pathname.substring(url.length + 1),
                createContext: async () => createContext?.(event),
                responseMeta,
                // eslint-disable-next-line @typescript-eslint/no-explicit-any
                onError: onError
            });
            const { status, headers, body } = httpResponse;
            // Set headers and cookies that were set using SvelteKit's `event.setHeaders` and `event.cookies.set`.
            for (const [key, value] of Object.entries(headersProxy)) {
                headers[key] = value;
            }
            // Current cookies from TRPC
            const currentCookiesHeaders = headers['Set-Cookie']?.split('; ').map((cookie) => {
                const name = cookie.split('=')[0];
                const value = cookie.split('=').at(1);
                return { name, value };
            }) ?? [];
            // Add / Replace cookies from SvelteKit
            for (const cookie of event.cookies.getAll()) {
                // Remove existing cookie
                const existingCookieIndex = currentCookiesHeaders.findIndex((c) => c.name === cookie.name);
                if (existingCookieIndex !== -1) {
                    currentCookiesHeaders.splice(existingCookieIndex, 1);
                }
                // Add the new cookie
                currentCookiesHeaders.push({
                    name: cookie.name,
                    value: cookie.value
                });
            }
            // Set "Set-Cookie" header
            headers['Set-Cookie'] = currentCookiesHeaders
                .map((cookie) => `${cookie.name}=${cookie.value}`)
                .join('; ');
            return new Response(body, { status, headers });
        }
        return resolve(event);
    };
}
